package TestAppController;

use base 'Terse::Controller';

sub hidden {

}

sub login :any {
	return 1;
}

sub auth_prevent :req(auth) :any {
	return 0;
}

sub auth_okay :req(auth) :get :post  {
	return 1;
}

sub hello :req(hello_world) :any {
	$_[1]->response->hello = "world";
}

sub hello_get :req(hello_world) :get {
	$_[1]->response->hello = "world 2";
}

sub hello_get_with_params :req(hello_world) :get :params(test => 'okay') {
	$_[1]->response->hello = "world 3";
}

1;
